import pytest

from galaxy.tool_util.deps.mulled.mulled_search import CondaSearch, get_package_hash, GitHubSearch, QuaySearch, run_command, singularity_search
from ..util import external_dependency_management


@external_dependency_management
def test_quay_search():
    t = QuaySearch("biocontainers")
    t.build_index()
    search1 = t.search_repository("adsfasdf", True)
    search2 = t.search_repository("bioconductor-gosemsim", True)
    assert search1 == []
    assert {'version': '2.2.0--0', 'package': 'bioconductor-gosemsim'} in search2


@external_dependency_management
@pytest.mark.skipif(run_command is None, reason="requires import from conda library")
def test_conda_search():
    t = CondaSearch('bioconda')
    search1 = t.get_json("asdfasdf")
    search2 = t.get_json("bioconductor-gosemsim")
    assert search1 == []
    assert search2['version'] == '2.2.0'
    assert search2['package'] == 'bioconductor-gosemsim'
    assert search2['build'] == '0'


@external_dependency_management
def test_github_search():
    t = GitHubSearch()
    search1 = t.process_json(t.get_json("adsfasdf"), "adsfasdf")
    search2 = t.process_json(t.get_json("bioconductor-gosemsim"), "bioconductor-gosemsim")
    assert search1 == []
    assert {'path': 'recipes/bioconductor-gosemsim/meta.yaml', 'name': 'meta.yaml'} in search2


@external_dependency_management
def test_get_package_hash():
    package_hash1 = get_package_hash(['bamtools', 'samtools'], {})
    package_hash2 = get_package_hash(['bamtools', 'samtools'], {'bamtools': '2.4.0', 'samtools': '1.3.1'})
    package_hash3 = get_package_hash(['abricate', 'abyss'], {'abricate': '0.4', 'abyss': '2.0.1'})
    assert package_hash1['package_hash'] == 'mulled-v2-0560a8046fc82aa4338588eca29ff18edab2c5aa'
    assert package_hash2['version_hash'] == 'c17ce694dd57ab0ac1a2b86bb214e65fedef760e'
    assert package_hash2['package_hash'] == 'mulled-v2-0560a8046fc82aa4338588eca29ff18edab2c5aa'
    assert package_hash3['version_hash'] == 'e21d1262f064e1e01b6b9fad5bea117928f31b38'
    assert package_hash3['package_hash'] == 'mulled-v2-cde36934a4704f448af44bf01deeae8d2832ca2e'


@external_dependency_management
def test_singularity_search():
    sing1 = singularity_search('mulled-v2-0560a8046fc82aa4338588eca29ff18edab2c5aa')
    sing2 = singularity_search('mulled-v2-19fa9431f5863b2be81ff13791f1b00160ed0852')
    assert sing1[0]['version'] in ['c17ce694dd57ab0ac1a2b86bb214e65fedef760e-0', 'fc33176431a4b9ef3213640937e641d731db04f1-0']
    assert sing1[1]['version'] in ['c17ce694dd57ab0ac1a2b86bb214e65fedef760e-0', 'fc33176431a4b9ef3213640937e641d731db04f1-0']
    assert sing2 == []



"""Logic for dealing with cwltool as an optional dependency.

Use this as the import interface for cwltool and just call
:func:`ensure_cwltool_available` before using any of the imported
functionality at runtime.
"""
import re
import warnings

warnings.filterwarnings("ignore", message=r"[\n.]DEPRECATION: Python 2", module="cwltool")

import requests

try:
    from cwltool import (
        main,
        workflow,
        job,
        process,
        pathmapper,
    )
except ImportError:
    main = None  # type: ignore
    workflow = None  # type: ignore
    job = None  # type: ignore
    process = None  # type: ignore
    pathmapper = None  # type: ignore

try:
    from cwltool.context import (
        getdefault,
        LoadingContext,
        RuntimeContext,
    )
    from cwltool.job import relink_initialworkdir
    from cwltool.stdfsaccess import StdFsAccess
except ImportError:
    getdefault = None  # type: ignore
    LoadingContext = None  # type: ignore
    relink_initialworkdir = None  # type: ignore
    RuntimeContext = None  # type: ignore
    StdFsAccess = None  # type: ignore

try:
    from cwltool import load_tool
    from cwltool.load_tool import (
        default_loader,
        resolve_and_validate_document,
    )
except ImportError:
    default_loader = None  # type: ignore
    load_tool = None  # type: ignore
    resolve_and_validate_document = None  # type: ignore


def _has_relax_path_checks_flag():
    """Return True if cwltool uses a flag to control path checks.

    Old cwltool uses the module global below to control whether
    it's strict about path checks. New versions use an attribute
    of LoadingContext.

    Once the version of cwltool required is new enough, we can remove
    this function and simplify the conditionals where it's used.
    """

    lc = LoadingContext()
    return hasattr(lc, "relax_path_checks")


try:
    from cwltool import command_line_tool
    if not _has_relax_path_checks_flag():
        command_line_tool.ACCEPTLIST_RE = command_line_tool.ACCEPTLIST_EN_RELAXED_RE
except ImportError:
    command_line_tool = None  # type: ignore

try:
    from cwltool.load_tool import resolve_and_validate_document
except ImportError:
    resolve_and_validate_document = None  # type: ignore

try:
    import shellescape
except ImportError:
    shellescape = None  # type: ignore

try:
    import schema_salad
    from schema_salad import (
        ref_resolver,
        sourceline,
    )
except ImportError:
    schema_salad = None  # type: ignore
    ref_resolver = None  # type: ignore
    sourceline = None  # type: ignore

needs_shell_quoting = re.compile(r"""(^$|[\s|&;()<>\'"$@])""").search

# if set to True, file format checking is not performed.
beta_relaxed_fmt_check = True


def ensure_cwltool_available():
    """Assert optional dependencies proxied via this module are available at runtime.

    Throw an ImportError with a description of the problem if they do not exist.
    """
    if main is None or workflow is None or shellescape is None:
        message = "This feature requires cwltool and dependencies to be available, they are not."
        if main is None:
            message += " cwltool is not unavailable."
        elif resolve_and_validate_document is None:
            message += " cwltool.load_tool.resolve_and_validate_document is unavailable - cwltool version is too old."
        if requests is None:
            message += " Library 'requests' unavailable."
        if shellescape is None:
            message += " Library 'shellescape' unavailable."
        if schema_salad is None:
            message += " Library 'schema_salad' unavailable."
        raise ImportError(message)


__all__ = (
    'default_loader',
    'ensure_cwltool_available',
    'getdefault',
    'load_tool',
    'LoadingContext',
    'main',
    'needs_shell_quoting',
    'pathmapper',
    'process',
    'ref_resolver',
    'relink_initialworkdir',
    'resolve_and_validate_document',
    'RuntimeContext',
    'schema_salad',
    'shellescape',
    'sourceline',
    'StdFsAccess',
    'workflow',
)



import re


def assert_has_text(output, text, n=None):
    """ Asserts specified output contains the substring specified by
    the argument text. The exact number of occurrences can be
    optionally specified by the argument n."""
    if n is None:
        assert output.find(text) >= 0, f"Output file did not contain expected text '{text}' (output '{output}')"
    else:
        matches = re.findall(re.escape(text), output)
        assert len(matches) == int(n), "Expected {} matches for '{}' in output file (output '{}'); found {}".format(n, text, output, len(matches))


def assert_not_has_text(output, text):
    """ Asserts specified output does not contain the substring
    specified by the argument text."""
    assert output.find(text) < 0, "Output file contains unexpected text '%s'" % text


def assert_has_line(output, line, n=None):
    """ Asserts the specified output contains the line specified by the
    argument line. The exact number of occurrences can be optionally
    specified by the argument n."""
    if n is None:
        match = re.search("^%s$" % re.escape(line), output, flags=re.MULTILINE)
        assert match is not None, f"No line of output file was '{line}' (output was '{output}') "
    else:
        matches = re.findall("^%s$" % re.escape(line), output, flags=re.MULTILINE)
        assert len(matches) == int(n), "Expected {} lines matching '{}' in output file (output was '{}'); found {}".format(n, line, output, len(matches))


def assert_has_n_lines(output, n):
    """Asserts the specified output contains ``n`` lines."""
    n_lines_found = len(output.splitlines())
    assert n_lines_found == int(n), f"Expected {n} lines in output, found {n_lines_found} lines"


def assert_has_text_matching(output, expression):
    """ Asserts the specified output contains text matching the
    regular expression specified by the argument expression."""
    match = re.search(expression, output)
    assert match is not None, "No text matching expression '%s' was found in output file." % expression


def assert_has_line_matching(output, expression):
    """ Asserts the specified output contains a line matching the
    regular expression specified by the argument expression."""
    match = re.search("^%s$" % expression, output, flags=re.MULTILINE)
    assert match is not None, "No line matching expression '%s' was found in output file." % expression
